% HEADER %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This is file spam/man/methods.Rd.                         %
% It is part of the R package spam,                         %
%  --> https://CRAN.R-project.org/package=spam              %
%  --> https://CRAN.R-project.org/package=spam64            %
%  --> https://git.math.uzh.ch/reinhard.furrer/spam         %
% by Reinhard Furrer [aut, cre], Florian Gerber [aut],      %
%    Roman Flury [aut], Daniel Gerber [ctb],                %
%    Kaspar Moesinger [ctb]                                 %
% HEADER END %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   


\name{spam methods}
\alias{methods.spam}
\title{Methods for Sparse Matrices}
\description{Methods without any additional parameters for sparse matrices.
}
\details{If a method for \code{spam} objects takes the same arguments,
  produces the intuitive output. We do not provide additional help
  pages.

  However, such methods are usually linked to a \code{xzy.spam}
  function, that could also be called directly.
	    }
\seealso{Corresponding base help functions.
}
\author{Reinhard Furrer}
\keyword{internal}
